// ArraysExt.js v.0.1 - Marco Di Mario (mkdm), 2016
// based on :
// 		Max Smirnov's Node Editor, v.0.65 - 2016
// 		litegraph.js library (c) Javi Agenjo http://tamats.com


(function () {

// PatternSelectArray
function PatternSelArray()
{	
	this.size = [160,70];
	this.minsize = [160,70];
	
	this.addInput("In","pointarray");
	this.addInput("Start_Idx","numarray");
	this.addInput("Qt_Max","numarray");
	this.addInput("Step","numarray");		
	
	this.addOutput("Out","pointarray");
	this.addOutput("Elems","number");
	this.addOutput("Source","pointarray");
	
	this.properties = { mode:["Pattern + Subset", "Pattern + Subset", "Subset + Pattern"],
		pattern:"",
		start_idx:[0],
		qt_max:[-1],
		step:[1] };
	
}

PatternSelArray.title = "PatternSelArray";
PatternSelArray.desc = "PatternSelArray";

PatternSelArray.prototype.onExecute = function()
{		
	/*
		source = the source array
		start_idx = index of the first element to pick
		numMaxElem = max number of elements that will be present in the resulting array
		step = step amount for the for..loop
	*/
	var getPointArraySection = function(source, start_idx, numMaxElem, step) {

		var result = new pointArray();
		
		if (source) {
			var sourceLength = source.getLength();
			for (var i = start_idx, cont = 0; i < sourceLength && cont < numMaxElem; i += step, cont++) {
				var source_elem = source.getElement(i);
				
				result.push(source_elem.data[0],
					source_elem.data[1],
					source_elem.data[2],
					source_elem.data[3],
					source_elem.data[4],
					source_elem.data[5],
					source_elem.data[6]);
			}
		}
		return result;
	}

	/*
		source = the source array
		pattern = string that define the selection pattern
			1 or + = pick element
			0 or - = skip element
	*/	
	var applyPatternToPointArray = function(source, pattern)
	{
		function ResultObj(pattern) {
			this.array = new pointArray();
			this.pattern = pattern;
		}

		var usePattern = false;
		var isMixedPattern = false;			
		var result = new ResultObj(pattern);
		
		if (pattern) {
			result.pattern = pattern.trim();
			
			// check the pattern
			if (result.pattern !== '') {
				// check if pattern contains valid characters
				usePattern = result.pattern.match('^[+\-0-1]+$');
				
				if (usePattern) {
					// check if pattern contains only include characters (+ or 1)
					usePattern = !result.pattern.match('^[+1]+$');
					
					if (usePattern) {
						// if pattern contains both include and exclude characters, then the pattrn will be used
						// otherwise it means that contains only exclude characters (- or 0), and therefore
						// the output pointarray list will be empty
						isMixedPattern = !result.pattern.match('^[\-0]+$');
					}
				}
			}
		}

		if (usePattern) {
			if (isMixedPattern) {
				var len = source.getLength();
				var patLen = result.pattern.length;
				var cn = 0;
				
				for (i = 0; i < len ; i++) {
					if ( result.pattern[cn] === '1' || result.pattern[cn] === '+') {
								
						var source_elem = source.getElement(i);
						
						result.array.push(source_elem.data[0],
							source_elem.data[1],
							source_elem.data[2],
							source_elem.data[3],
							source_elem.data[4],
							source_elem.data[5],
							source_elem.data[6]);
					}
					cn = (cn < (patLen - 1)) ? (cn + 1) : 0;
				}
			}
		} else {
			// elaborate the whole source pointarray
			result.pattern = '';
			result.array = source;		
		}

		return result;
	}

	// Business logic
	var source = this.getInputData(0, new pointArray());
	
	this.properties.start_idx = this.getInputData(1, this.properties.start_idx);
	this.properties.qt_max = this.getInputData(2, this.properties.qt_max);
	this.properties.step = this.getInputData(3, this.properties.step);

	if (this.properties.step[0] <= 0) this.properties.step[0] = 1;
	
	var patternedArray;
	var outputIntermediate;
	var outIntermLength;
	
	if (this.properties.mode[0] === "Pattern + Subset") {
		patternedArray = applyPatternToPointArray(source, this.properties.pattern);		
		outputIntermediate = patternedArray.array;		
		outIntermLength = outputIntermediate.getLength();
	} else {
		outputIntermediate = source;
		outIntermLength = source.getLength();
	}

	if ((this.properties.start_idx[0] < 0) || (this.properties.start_idx[0] > (outIntermLength - 1))) this.properties.start_idx[0] = 0;
	
	if (this.properties.qt_max[0] < 0) this.properties.qt_max[0] = outIntermLength;
	
	outputIntermediate = getPointArraySection(outputIntermediate,
						this.properties.start_idx[0],
						this.properties.qt_max[0],
						this.properties.step[0]);

	if (this.properties.mode[0] === "Subset + Pattern") {
		patternedArray = applyPatternToPointArray(outputIntermediate, this.properties.pattern);
		
		outputIntermediate = patternedArray.array;
		
		outIntermLength = outputIntermediate.getLength();
	}
						
	this.properties.pattern = patternedArray.pattern;
	patternedArray = null;
	
	this.outputs[0].label = "Out (" + outputIntermediate.getLength() + ")";
	this.outputs[1].label = "Elems (" + outputIntermediate.getLength() + ")";
	this.inputs[0].label = "In (" + source.getLength() + ")";
	
	this.setOutputData(0, outputIntermediate);
	this.setOutputData(1, outputIntermediate.getLength());
	this.setOutputData(2, source);
	
}

LiteGraph.registerNodeType("ArraysExt/PatternSelArray", PatternSelArray);


// ArrayShuffler
    function ArrayShuffler()
    {
        this.size = [160, 40];
        this.minsize = [160, 40];

        this.addInput("In", "pointarray");

        this.addOutput("Out", "pointarray");

    }

    ArrayShuffler.title = "ArrayShuffler";
    ArrayShuffler.desc = "ArrayShuffler";

    ArrayShuffler.prototype.onExecute = function ()
    {
        function shuffle(source) {
            var rndIdx, element1, element2, i;
            var source_len = source.getLength();

            for (i = source_len; i; i -= 1) {
                rndIdx = Math.floor(Math.random() * i);

                element1 = source.getElement(i - 1);
                element2 = source.getElement(rndIdx);

//                source.copyElementDataTo(element2, i - 1);
                source.data[(i-1)*source.recordLength]=element2.data[0];
                source.data[((i-1)*source.recordLength)+1]=element2.data[1];
                source.data[((i-1)*source.recordLength)+2]=element2.data[2];
                source.data[((i-1)*source.recordLength)+3]=element2.data[3];
                source.data[((i-1)*source.recordLength)+4]=element2.data[4];
                source.data[((i-1)*source.recordLength)+5]=element2.data[5];
                source.data[((i-1)*source.recordLength)+6]=element2.data[6];
//                source.copyElementDataTo(element1, rndIdx);
                source.data[rndIdx*source.recordLength]=element1.data[0];
                source.data[(rndIdx*source.recordLength)+1]=element1.data[1];
                source.data[(rndIdx*source.recordLength)+2]=element1.data[2];
                source.data[(rndIdx*source.recordLength)+3]=element1.data[3];
                source.data[(rndIdx*source.recordLength)+4]=element1.data[4];
                source.data[(rndIdx*source.recordLength)+5]=element1.data[5];
                source.data[(rndIdx*source.recordLength)+6]=element1.data[6];
            }
        }

        var source = this.getInputData(0, new pointArray());
        var output = source.clone();

        shuffle(output);

        this.inputs[0].label = "In (" + source.getLength() + ")";

        this.setOutputData(0, output);
    }

    ArrayShuffler.prototype.getExtraMenuOptions = function (graphcanvas) {
        var that = this,
                thatgraph = this.graph;
        return [{content: lang.getTranslation("Update"),
                callback: function () {
                    thatgraph.setisChangedFlag(that.id);
                }
            }];
    }

    LiteGraph.registerNodeType("ArraysExt/ArrayShuffler", ArrayShuffler);

// ArrayFlipper
    function ArrayFlipper()
    {
        this.size = [160, 40];
        this.minsize = [160, 40];

        this.addInput("In", "pointarray");

        this.addOutput("Out", "pointarray");

    }

    ArrayFlipper.title = "ArrayFlipper";
    ArrayFlipper.desc = "ArrayFlipper";

    ArrayFlipper.prototype.onExecute = function ()
    {
        function flip(source) {
            var output = new pointArray();

            var source_len = source.getLength();

            for (i = (source_len - 1); i >= 0; i--) {

                var elemSource = source.getElement(i);

                output.push(elemSource.data[0],
                        elemSource.data[1],
                        elemSource.data[2],
                        elemSource.data[3],
                        elemSource.data[4],
                        elemSource.data[5],
                        elemSource.data[6])
            }

            return output;
        }

        var source = this.getInputData(0, new pointArray());

        var output = flip(source);

        this.inputs[0].label = "In (" + source.getLength() + ")";

        this.setOutputData(0, output);
    }

    LiteGraph.registerNodeType("ArraysExt/ArrayFlipper", ArrayFlipper);

// ArrayLogger
    function ArrayLogger()
    {
        this.size = [120, 30];
        this.minsize = [120, 30];

        this.addInput("In", "pointarray");

        this.properties = {_log: ""};

    }

    ArrayLogger.title = "ArrayLogger";
    ArrayLogger.desc = "ArrayLogger";

    ArrayLogger.prototype.onExecute = function ()
    {

        var output = "";
        var source = this.getInputData(0, new pointArray());

        var sourceLength = source.getLength();

        for (i = 0; i < sourceLength; i++) {
            var element = source.getElement(i);

            output += element.data[0]
            output += " "
            output += element.data[1]
            output += " "
            output += element.data[2]
            output += " "
            output += element.data[3]
            output += " "
            output += element.data[4]
            output += " "
            output += element.data[5]
            output += " "
            output += element.data[6]
            output += "\n"
        }
                    output += "xlength";
                    output += source.xlength;
                    output += "\n";
                    output += "ylength";
                    output += source.ylength;
                    output += "\n";
                    output += "zlength";
                    output += source.zlength;
                    output += "\n";
        

        this.inputs[0].label = "In (" + source.getLength() + ")";

        this.properties["_log"] = output;

    }

    LiteGraph.registerNodeType("ArraysExt/ArrayLogger", ArrayLogger);

    function idxSelect()
    {
        this.properties = {indices:[0]}
    }

    idxSelect.title = "idxSelect";
    idxSelect.desc = "idxSelect";

    idxSelect.prototype.getExtraMenuOptions = function ()
    {
        var that = this;
        return [
            {content: lang.getTranslation("Numbers"), callback: function () {
                    if (that.inputs[0].type !== "numarray") {
                        var n = that.inputs.length;
                        for (var i = that.inputs.length - 1; i >= 0; i--)
                            that.removeInput(i, "");
//                        for (var i = 0; i < n; i++)
                            that.addInput("In", "numarray");
                            that.addInput("Idx", "numarray");
                        that.removeOutput(0, "");
                        that.addOutput("Out", "numarray");
                    }
                }},
            {content: lang.getTranslation("Points"), callback: function () {
                    if (that.inputs[0].type !== "pointarray") {
                        var n = that.inputs.length;
                        for (var i = that.inputs.length - 1; i >= 0; i--)
                            that.removeInput(i, "");
//                        for (var i = 0; i < n; i++)
                            that.addInput("InPts", "pointarray");
                            that.addInput("Idx", "numarray");
                        that.removeOutput(0, "");
                        that.addOutput("Out", "pointarray");
                    }
                }},
            {content: lang.getTranslation("Objects"), callback: function () {
                    if (that.inputs[0].type !== "objectlist") {
                        var n = that.inputs.length;
                        for (var i = that.inputs.length - 1; i >= 0; i--)
                            that.removeInput(i, "");
//                        for (var i = 0; i < n; i++)
                            that.addInput("InObj", "objectlist");
                            that.addInput("idx", "numarray");
                        that.removeOutput(0, "");
                        that.addOutput("Out", "objectlist");
                    }
                }}
        ];
    }

    idxSelect.prototype.onAdded = function ()
    {
            if (!this.inputs.length){        
            this.addInput("InPts", "pointarray");
            this.addInput("Idx", "numarray");
            }else{
                if (this.inputs.length===0){        
                this.addInput("InPts", "pointarray");
                this.addInput("Idx", "numarray");
            }
                
            }
        
            if(!this.outputs.length){ this.addOutput("Out", "pointarray");}
            else{if(this.outputs.length===0) this.addOutput("Out", "pointarray");}
        this.setDirtyCanvas(true);
    }
    idxSelect.prototype.onExecute = function ()
    {
        if (this.inputs[0].type === "numarray") {
            var out = [];
            var source = this.getInputData(0, "numarray");
            this.properties.indices = this.getInputData(1, this.properties.indices);
            for (var i = 0; i < this.properties.indices.length; i++) {
                var idx = Math.round(this.properties.indices[i]);
                if (idx >= 0 && idx < source.length) out.push(source[idx]);
            }
        }
        if (this.inputs[0].type === "pointarray") {
        var source = this.getInputData(0, new pointArray());
        this.properties.indices = this.getInputData(1, this.properties.indices);
        var out = new pointArray(false);
        var sourceLen=source.getLength();
        for(var i=0; i<this.properties.indices.length;i++){
            var idx=Math.round(this.properties.indices[i]);
            if(idx>=0&&idx<sourceLen){
            var elemSource = source.getElement(idx);
                out.push(elemSource.data[0],
                        elemSource.data[1],
                        elemSource.data[2],
                        elemSource.data[3],
                        elemSource.data[4],
                        elemSource.data[5],
                        elemSource.data[6]);
                        }
        }
        }
        if (this.inputs[0].type === "objectlist") {
            var out = moi.geometryDatabase.createObjectList();
        var source = this.getInputData(0, moi.geometryDatabase.createObjectList());
        this.properties.indices = this.getInputData(1, this.properties.indices);
        for(var i=0; i<this.properties.indices.length;i++){
            var idx=Math.round(this.properties.indices[i]);
            if(idx>=0&&idx<source.length){out.addObject(source.item(idx));}            
            }
        }
        this.setOutputData(0, out);
    }

    LiteGraph.registerNodeType("ArraysExt/idxSelect", idxSelect);
//////
    function idxReplace()
    {
//        this.size = [160, 40];
//        this.minsize = [160, 40];

        this.addInput("In", "numarray");
        this.addInput("With", "numarray");
        this.addInput("Idx", "numarray");

        this.addOutput("Out", "numarray");
        this.properties = { indices:[0]}

    }

    idxReplace.title = "idxReplace";
    idxReplace.desc = "idxReplace";

    idxReplace.prototype.onExecute = function ()
    {

        var source = this.getInputData(0, "numarray");
        var rep = this.getInputData(1, "numarray");
        this.properties.indices = this.getInputData(2, this.properties.indices);
        var output = [];
        for(var n=0;n<source.length;n++)(output.push(source[n])); //copy of the source
        for(var i=0; i<this.properties.indices.length;i++){
//            var idx=Math.round(this.properties.indices[i]);
            if(this.properties.indices[i]>=0&&this.properties.indices[i]<source.length){
   
                output[this.properties.indices[i]]=rep[i];
                        }
        }
        this.setOutputData(0, output);
    }

    LiteGraph.registerNodeType("ArraysExt/idxReplace", idxReplace);

    function shiftDublicate()
    {
//        this.size = [160, 40];
//        this.minsize = [160, 40];

        this.addInput("In", "numarray");
        this.addInput("Shift", "numarray");
        this.addInput("x N", "numarray");

        this.addOutput("Out", "numarray");
        this.properties = { inp:[0],shift:[0],N:[1]}

    }

    shiftDublicate.title = "shiftDublicate";
    shiftDublicate.desc = "shiftDublicate";

    shiftDublicate.prototype.onExecute = function ()
    {

        var source = this.getInputData(0, this.properties.inp);
        this.properties.shift = this.getInputData(1, this.properties.shift);
        this.properties.N = this.getInputData(2, this.properties.N);
        var output = [];

        for(var i=0; i<this.properties.N[0];i++){
                    for(var j=0; j<source.length;j++){
                output.push(source[j]+i*this.properties.shift);
                }
        }
        this.outputs[0].label = output.length;
        this.setOutputData(0, output);
    }

    LiteGraph.registerNodeType("ArraysExt/shiftDublicate", shiftDublicate);
//
// ###################
function reMap()
{
	this.internal = {}
	this.properties = { no:[5]};
        this.addInput("Array","numarray");
	this.addInput("N values","numarray");
	this.addOutput("Interp", "numarray");
}

 reMap.title = "reMap";
 reMap.desc = "reMap";

 reMap.prototype.onAdded = function()
{
}


 reMap.prototype.onPropertyChange = function( property, value )
{
};

reMap.prototype.onExecute = function()
{
             var inpArray = this.getInputData(0, new Array());
             this.properties.no = this.getInputData(1, this.properties.no);
             var sM = new splineMap(inpArray);
             var out=sM.calc(this.properties.no[0]);
             	this.setOutputData(0, out);
}

LiteGraph.registerNodeType("ArraysExt/reMap",  reMap);
// Concat
    function reMap2()
    {
        this.labels = "abcdef";
        this.internal = {a: [], b: []};
        this.properties = { no:[5]};
        this.zero=[0];
    }

    reMap2.title = "reMap2";
    reMap2.desc = "reMap2";

    reMap2.prototype.getExtraMenuOptions = function ()
    {
        var that = this;
        return [
            {content: lang.getTranslation("Numbers"), callback: function () {
                    if (that.inputs[0].type !== "numarray") {
                        var n = that.inputs.length;
                        for (var i = that.inputs.length - 1; i >= 0; i--)
                            that.removeInput(i, "");
                            that.addInput("Num", "numarray");
                            that.addInput("N Values", "numarray");                            
                            that.removeOutput(0, "");
                            that.addOutput("Out", "numarray");
                    }
                }},
            {content: lang.getTranslation("Points"), callback: function () {
                    if (that.inputs[0].type !== "pointarray") {
                        var n = that.inputs.length;
                        for (var i = that.inputs.length - 1; i >= 0; i--)
                            that.removeInput(i, "");
                            that.addInput("Pts", "pointarray");
                            that.addInput("N Values", "numarray");
                        that.removeOutput(0, "");
                        that.addOutput("Out", "pointarray");
                    }
                }}
        ];
    }

    reMap2.prototype.onAdded = function ()
    {
/*        for (var i = 0; i < this.labels.length; i++)
            if (this.internal[this.labels.charAt(i)] && this.inputs.length === i)        
                this.addInput("Num", "numarray");*/
            if(this.inputs.length===0){
                this.addInput("Array","numarray");
        	this.addInput("N values","numarray");
                }
            if(this.outputs.length===0) this.addOutput("Out", "numarray");
        this.setDirtyCanvas(true);
    }

    reMap2.prototype.onExecute = function ()
    {
        if (this.inputs[0].type === "numarray") {
            var out=[];
             var inpArray = this.getInputData(0, new Array());
             this.properties.no = this.getInputData(1, this.properties.no);
             var sM = new splineMap(inpArray);
             out=sM.calc(this.properties.no[0]);
             this.setOutputData(0, out);
        }
        if (this.inputs[0].type === "pointarray") {
            var out = new pointArray();
            var inpArray = this.getInputData(0, new Array());
            this.properties.no = this.getInputData(1, this.properties.no);
             //eval all x,y and z coordinates to a array
             var x=[],y=[],z=[],rx=[],ry=[],rz=[],s=[];
            for(var i=0; i<inpArray.length;i++){
/*             var pt=inpArray.getPoint(i);
             x.push(pt.x);
             y.push(pt.y);
             z.push(pt.z);*/
                var pt=inpArray.getElement(i);
                x.push(pt.data[0]);
                y.push(pt.data[1]);
                z.push(pt.data[2]);
                rx.push(pt.data[3]);
                ry.push(pt.data[4]);
                rz.push(pt.data[5]);
                s.push(pt.data[6]);
         }
            var xmap = new splineMap(x);
            var ymap = new splineMap(y);
            var zmap = new splineMap(z);
            var rxmap = new splineMap(rx);
            var rymap = new splineMap(ry);
            var rzmap = new splineMap(rz);
            var smap = new splineMap(s);
        // xremaped has now new interpolated values
             var xremaped=xmap.calc(this.properties.no[0]);
             var yremaped=ymap.calc(this.properties.no[0]);
             var zremaped=zmap.calc(this.properties.no[0]);
             var rxremaped=rxmap.calc(this.properties.no[0]);
             var ryremaped=rymap.calc(this.properties.no[0]);
             var rzremaped=rzmap.calc(this.properties.no[0]);
             var sremaped=smap.calc(this.properties.no[0]);
        // now creating the new points
            for(var i=0; i<xremaped.length;i++){
            out.push(xremaped[i],yremaped[i],zremaped[i],rxremaped[i],ryremaped[i],rzremaped[i],1);
                }
        }

        this.setOutputData(0, out);
    }

    LiteGraph.registerNodeType("ArraysExt/reMap2", reMap2);
    
    ///
    // bundlePts
    function bundleArrays()
    {
        this.labels = "abcdef";
//        this.size = [60, 20];
//        this.dynamicInternal = ["a", "b", "c", "d", "e", "f"];
        this.internal = {a: [], b: []};
//        this.addOutput("Out", "numarray");
//this.properties={xl:0,yl:0,zl:0}
this.properties={}
//        this.zero=[0];
    }

    bundleArrays.title = "bundleArray";
    bundleArrays.desc = "bundleArray";

    bundleArrays.prototype.getExtraMenuOptions = function ()
    {
        var that = this;
        return [
            {content: lang.getTranslation("Numbers"), callback: function () {
                    if (that.inputs[0].type !== "numarray") {
                        var n = that.inputs.length;
                        for (var i = that.inputs.length - 1; i >= 0; i--)
                            that.removeInput(i, "");
                        for (var i = 0; i < n; i++)
                            that.addInput("Num", "numarray");
                        that.removeOutput(0, "");
                        that.addOutput("Out", "numarray");
                    }
                }},
            {content: lang.getTranslation("Points"), callback: function () {
                    if (that.inputs[0].type !== "pointarray") {
                        var n = that.inputs.length;
                        for (var i = that.inputs.length - 1; i >= 0; i--)
                            that.removeInput(i, "");
                        for (var i = 0; i < n; i++)
                            that.addInput("Pts", "pointarray");
                        that.removeOutput(0, "");
                        that.addOutput("Out", "pointarray");
                    }
                }},
            {content: lang.getTranslation("Objects"), callback: function () {
                    if (that.inputs[0].type !== "objectlist") {
                        var n = that.inputs.length;
                        for (var i = that.inputs.length - 1; i >= 0; i--)
                            that.removeInput(i, "");
                        for (var i = 0; i < n; i++)
                            that.addInput("Obj", "objectlist");
                        that.removeOutput(0, "");
                        that.addOutput("Out", "objectlist");
                    }
                }}
        ];
    }

    bundleArrays.prototype.onGetInputs = function ()
    {var inpName="";
        var list = [];
        if(this.inputs[0].type==="numarray") inpName="Num";
        if(this.inputs[0].type==="pointarray") inpName="Pts";
        if(this.inputs[0].type==="objectlist") inpName="Obj";

        list.push([inpName, this.inputs[0].type, {locked: true}]);

        return list;
    }

    bundleArrays.prototype.onInputAdded = function (inp)
    {
        this.internal[inp.name] = 1;
        for (var i in this.inputs)
            this.inputs[i].locked = true;
        if (i > 1)
            this.inputs[i].locked = false;
    }

    bundleArrays.prototype.onInputRemoved = function (slot, name)
    {
        if (this.internal[name])
            delete this.internal[name];
        for (var i in this.inputs)
            this.inputs[i].locked = true;
        if (this.inputs.length > 2)
            this.inputs[i].locked = false;
    }

    bundleArrays.prototype.onAdded = function ()
    {
        for (var i = 0; i < this.labels.length; i++)
            if (this.internal[this.labels.charAt(i)] && this.inputs.length === i)
//            if (this.inputs.length === i)
                this.addInput("Pts", "pointarray");
        
            if(this.outputs.length===0) this.addOutput("Out", "pointarray");
        this.setDirtyCanvas(true);
    }

    bundleArrays.prototype.onExecute = function ()
    {
        if (this.inputs[0].type === "numarray") {
            var out = [];
            var inps=[];
            var minL=10000000;
            for (var i = 0; i < this.inputs.length; i++)
            {
                inps[i] = this.getInputData(i, "numarray");
                if(inps[i].length<minL)minL=inps[i].length;
            }
            var inpsL=inps.length;
            for (var k = 0; k < minL; k++){
            for (var j = 0; j < inpsL; j++){
            out.push(inps[j][k]);
                }
            }
        }
        if (this.inputs[0].type === "pointarray") {
            var inPts=[];
            var out = new pointArray();
            var minL=10000000;
            for (var i = 0; i < this.inputs.length; i++)
            {
                inPts.push(this.getInputData(i, new pointArray()));
                if(inPts[i].getLength()<minL)minL=inPts[i].getLength();
            }
//            for (var j = 0; j < inPts[0].getLength(); j++){
            for (var j = 0; j < minL; j++){
                for (var k = 0; k < inPts.length; k++){
                out.concat(inPts[k].getElement(j));
            }
            out.xlength=inPts.length;    
            out.ylength=1;            
//            out.zlength=inPts[0].getLength();
            out.zlength=minL;
            }            
        }
        if (this.inputs[0].type === "objectlist") {
            var out = moi.geometryDatabase.createObjectList();
            var inps=[];
            var minL=10000000;
            for (var i = 0; i < this.inputs.length; i++)
            {
                inps[i] = this.getInputData(i, moi.geometryDatabase.createObjectList());
                if(inps[i].length<minL)minL=inps[i].length;
            }
            var inpsL=inps.length;
            for (var k = 0; k < minL; k++){
            for (var j = 0; j < inpsL; j++){
                    out.addObject(inps[j].item(k));
                }
            }
        }
        this.setOutputData(0, out);
    }

    LiteGraph.registerNodeType("ArraysExt/bundleArray", bundleArrays);
    ///

})();