// MoI Nodeedit v.0.6 - Max Smirnov, 2016
// litegraph.js library (c) Javi Agenjo http://tamats.com
// Info nodes Karsten Marquas

(function () {

// ############ bBox ############
    function bBox()
    {
        this.addInput("In", "objectlist");

        this.addOutput("x-length", "numarray");
        this.addOutput("y-length", "numarray");
        this.addOutput("z-length", "numarray");

        this.bbox = moi.VectorMath.createBoundingBox();
    }

    bBox.title = "BoundingBox-Infos";
    bBox.desc = "BoundingBox-Infos";

    bBox.prototype.onExecute = function ()
    {
        var inObj = this.getInputData(0, moi.geometryDatabase.createObjectList());
//       this.bbox.addB[i]oundingBox(inObj.getBoundingBox());
        this.bbox = inObj.getHighAccuracyBoundingBox();
        var x=[this.bbox.xLength],y=[this.bbox.yLength],z=[this.bbox.zLength];
        this.outputs[0].label = x[0].toFixed(2);
        this.setOutputData(0, x);
        this.outputs[1].label = y[0].toFixed(2);
        this.setOutputData(1, y);
        this.outputs[2].label = z[0].toFixed(2);
        this.setOutputData(2, z);
    }
    bBox.prototype.onPropertyChange = function ()
    {

        //       this.outputs[0].label = this.bbox.xLength.toFixed(2);
        //       this.outputs[1].label = this.bbox.yLength.toFixed(2);
        //       this.outputs[2].label = this.bbox.zLength.toFixed(2);
    }

    LiteGraph.registerNodeType("Infos/bBox", bBox);

 // ############ bBox ############
    function crv_length()
    {
        this.addInput("In", "objectlist");
        this.addOutput("out", "objectlist");
        this.addOutput("length", "numarray");
    }

    crv_length.title = "Curve Length";
    crv_length.desc = "Curve Length";

    crv_length.prototype.onExecute = function ()
    {
        var inObj = this.getInputData(0, moi.geometryDatabase.createObjectList());
        var out = moi.geometryDatabase.createObjectList();
        var len=[];
        for(var i=0;i<inObj.length;i++)
        {
          if(inObj.item(i).type===1||inObj.item(i).type===2)
          {
          len.push(inObj.item(i).getLength());
          out.addObject(inObj.item(i).clone());
          } 
        }
        this.setOutputData(0, out);
        if(len.length>0) this.outputs[1].label = len[0].toFixed(2);
        this.setOutputData(1, len);
    }
    crv_length.prototype.onPropertyChange = function ()
    {

        //       this.outputs[0].label = this.bbox.xLength.toFixed(2);
        //       this.outputs[1].label = this.bbox.yLength.toFixed(2);
        //       this.outputs[2].label = this.bbox.zLength.toFixed(2);
    }

    LiteGraph.registerNodeType("Infos/crv_length", crv_length);   
    //Curvature
    function Curvature()
    {
        this.addInput("Curve", "objectlist");
        this.addOutput("Out", "objectlist");
        this.addOutput("min R", "numarray");
        this.addOutput("max R", "numarray");
//        this.addOutput("naked Edges", "objectlist");
        this.properties = {analyse_as: ["Curvature", "Curvature", "Radius"], NumItems: 100, scalefactor: 0.1};
//        this.tempObjects = moi.geometryDatabase.createObjectList();
        this.rbID = new Array();
    }
    Curvature.title = "Curvature";
    Curvature.desc = "Curvature";
    Curvature.prototype.onAdded = function ()
    {
        this.rbID = createRainbow();
    }
    Curvature.prototype.onExecute = function ()
    {

        var output = moi.geometryDatabase.createObjectList();
        var inObjs = this.getInputData(0, moi.geometryDatabase.createObjectList());
        var inCurve = inObjs.getCurves();
        var minCurvature;
        var maxCurvature;
        var curvatureArray = new Array();
        if (inCurve.length > 0)
        {
            var crv = inCurve.item(0);
            var pointfactory = moi.command.createFactory('point');
            pointfactory.setInput(0, crv.getStartPt());
            var startPtObject = pointfactory.calculate();
            pointfactory.cancel();
            var acFactory = moi.command.createFactory('arraycurve'); ///
            acFactory.setInput(0, startPtObject);	// object (list)
            acFactory.setInput(1, crv);		// path
            acFactory.setInput(2, 'NumItems');		// array mode
            acFactory.setInput(3, this.properties.NumItems);
            var arrayed_pts = acFactory.calculate();

            var pts = new Array();
            pts.push(startPtObject.item(0).pt);
            for (var iPts = 0; iPts < arrayed_pts.length; ++iPts)
            {
                pts.push(arrayed_pts.item(iPts).pt);
            }
            var vecA = new Array();
            var vecB = new Array();
            var vecC = new Array();
            var ptA = new Array();
            var ptB = new Array();
            var ptC = new Array();
            var dA = new Array();
            var dB = new Array();
            var dC = new Array();
            for (var i = 1; i < pts.length - 1; i++)
            {
                var idx = i - 1;
                vecA.push(moi.vectorMath.createPoint(pts[i].x - pts[i - 1].x, pts[i].y - pts[i - 1].y, pts[i].z - pts[i - 1].z)); //Richtungsvektor P0-P1
                vecB.push(moi.vectorMath.createPoint(pts[i + 1].x - pts[i].x, pts[i + 1].y - pts[i].y, pts[i + 1].z - pts[i].z)); //Richtungsvektor P1-P2
                vecC.push(crossProd(vecA[idx], vecB[idx])); 	//Normalenvektor der Ebene P0,P1,P2 und Richtungsvektor des Schnitts
                ptA.push(addVec(scaleVec(vecA[idx], 0.5), pts[i - 1])); 	//Punkt des ersten Ebene
                ptB.push(addVec(scaleVec(vecB[idx], 0.5), pts[i]));
                //        ptC.push(pts[i]));


                dA.push(dotProd(vecA[idx], ptA[idx]));
                dB.push(dotProd(vecB[idx], ptB[idx]));
                dC.push(dotProd(vecC[idx], pts[i]));
            }
            for (var i = 0; i < vecA.length; i++)
            {
                var DetA = vecA[i].x * vecB[i].y * vecC[i].z + vecA[i].y * vecB[i].z * vecC[i].x + vecA[i].z * vecB[i].x * vecC[i].y - vecA[i].z * vecB[i].y * vecC[i].x - vecA[i].y * vecB[i].x * vecC[i].z - vecA[i].x * vecB[i].z * vecC[i].y;
                var DetX = dA[i] * vecB[i].y * vecC[i].z + vecA[i].y * vecB[i].z * dC[i] + vecA[i].z * dB[i] * vecC[i].y - vecA[i].z * vecB[i].y * dC[i] - vecA[i].y * dB[i] * vecC[i].z - dA[i] * vecB[i].z * vecC[i].y;
                var DetY = vecA[i].x * dB[i] * vecC[i].z + dA[i] * vecB[i].z * vecC[i].x + vecA[i].z * vecB[i].x * dC[i] - vecA[i].z * dB[i] * vecC[i].x - dA[i] * vecB[i].x * vecC[i].z - vecA[i].x * vecB[i].z * dC[i];
                var DetZ = vecA[i].x * vecB[i].y * dC[i] + vecA[i].y * dB[i] * vecC[i].x + dA[i] * vecB[i].x * vecC[i].y - dA[i] * vecB[i].y * vecC[i].x - vecA[i].y * vecB[i].x * dC[i] - vecA[i].x * dB[i] * vecC[i].y;
                if (DetA != 0)
                {
                    ptC.push(moi.vectorMath.createPoint(DetX / DetA, DetY / DetA, DetZ / DetA));
                } else {
                    ptC.push(moi.vectorMath.createPoint(pts[i + 1].x, pts[i + 1].y, pts[i + 1].z));
                }

            }
            //Lagrange interpolation for startcurvature with fixed values the Indices are the "x"-values (t-values) here - 0 is the variable of the startpoint
            var dx_1 = ptC[0].x - pts[1].x;
            var dy_1 = ptC[0].y - pts[1].y;
            var dz_1 = ptC[0].z - pts[1].z;
            var dx_2 = ptC[1].x - pts[2].x;
            var dy_2 = ptC[1].y - pts[2].y;
            var dz_2 = ptC[1].z - pts[2].z;
            var dx_3 = ptC[2].x - pts[3].x;
            var dy_3 = ptC[2].y - pts[3].y;
            var dz_3 = ptC[2].z - pts[3].z;
            var dx_0 = dx_1 * (0 - 2) * (0 - 3) / ((1 - 2) * (1 - 3)) + dx_2 * (0 - 1) * (0 - 3) / ((2 - 1) * (2 - 3)) + dx_3 * (0 - 1) * (0 - 2) / ((3 - 1) * (3 - 2));
            var dy_0 = dy_1 * (0 - 2) * (0 - 3) / ((1 - 2) * (1 - 3)) + dy_2 * (0 - 1) * (0 - 3) / ((2 - 1) * (2 - 3)) + dy_3 * (0 - 1) * (0 - 2) / ((3 - 1) * (3 - 2));
            var dz_0 = dz_1 * (0 - 2) * (0 - 3) / ((1 - 2) * (1 - 3)) + dz_2 * (0 - 1) * (0 - 3) / ((2 - 1) * (2 - 3)) + dz_3 * (0 - 1) * (0 - 2) / ((3 - 1) * (3 - 2));
            var ptCstart = moi.VectorMath.createPoint(
                    dx_0 + pts[0].x,
                    dy_0 + pts[0].y,
                    dz_0 + pts[0].z
                    );
            //Lagrange interpolation for endcurvature with fixed values the Indices are the "x"-values (t-values) here - 0 is the variable of the startpoint
            var idxC1 = ptC.length - 3;
            var idxC2 = ptC.length - 2;
            var idxC3 = ptC.length - 1;
            var idxpts1 = idxC1 + 1;
            var idxpts2 = idxC2 + 1;
            var idxpts3 = idxC3 + 1;
            var dx_u = ptC[idxC1].x - pts[idxpts1].x;
            var dy_u = ptC[idxC1].y - pts[idxpts1].y;
            var dz_u = ptC[idxC1].z - pts[idxpts1].z;
            var dx_v = ptC[idxC2].x - pts[idxpts2].x;
            var dy_v = ptC[idxC2].y - pts[idxpts2].y;
            var dz_v = ptC[idxC2].z - pts[idxpts2].z;
            var dx_w = ptC[idxC3].x - pts[idxpts3].x;
            var dy_w = ptC[idxC3].y - pts[idxpts3].y;
            var dz_w = ptC[idxC3].z - pts[idxpts3].z;
            var dx_e = dx_u * (4 - 2) * (4 - 3) / ((1 - 2) * (1 - 3)) + dx_v * (4 - 1) * (4 - 3) / ((2 - 1) * (2 - 3)) + dx_w * (4 - 1) * (4 - 2) / ((3 - 1) * (3 - 2));
            var dy_e = dy_u * (4 - 2) * (4 - 3) / ((1 - 2) * (1 - 3)) + dy_v * (4 - 1) * (4 - 3) / ((2 - 1) * (2 - 3)) + dy_w * (4 - 1) * (4 - 2) / ((3 - 1) * (3 - 2));
            var dz_e = dz_u * (4 - 2) * (4 - 3) / ((1 - 2) * (1 - 3)) + dz_v * (4 - 1) * (4 - 3) / ((2 - 1) * (2 - 3)) + dz_w * (4 - 1) * (4 - 2) / ((3 - 1) * (3 - 2));
            var ptCend = moi.VectorMath.createPoint(
                    dx_e + pts[pts.length - 1].x,
                    dy_e + pts[pts.length - 1].y,
                    dz_e + pts[pts.length - 1].z
                    );

            var analysePts = new Array();
            var scp;
            if (this.properties.analyse_as[0] == "Curvature")
            {/// Startpoint
                var d_vec = moi.vectorMath.createPoint(pts[0].x - ptCstart.x, pts[0].y - ptCstart.y, pts[0].z - ptCstart.z);
                var magn = magnitude(d_vec);
                minCurvature = 1 / magn;
                maxCurvature = minCurvature;
                curvatureArray.push(minCurvature);
                var sf = this.properties.scalefactor * 1000.0;
                if (magn > 0.002)
                {
                    var sd_vec = moi.vectorMath.createPoint((d_vec.x * sf / (magn * magn)), (d_vec.y * sf / (magn * magn)), (d_vec.z * sf / (magn * magn)));
                    scp = moi.vectorMath.createPoint(sd_vec.x + pts[0].x, sd_vec.y + pts[0].y, sd_vec.z + pts[0].z);
                } else
                {
                    scp = moi.vectorMath.createPoint(pts[0].x, pts[0].y, pts[0].z);
                }
                analysePts.push(scp);


                //////// the normally calculated    
                for (var n = 0; n < ptC.length; n++)
                {
                    var d_vec = moi.vectorMath.createPoint(pts[n + 1].x - ptC[n].x, pts[n + 1].y - ptC[n].y, pts[n + 1].z - ptC[n].z);
                    var magn = magnitude(d_vec);
                    //    var sf = this.properties.scalefactor * 1000.0;
                    var Curvature = 1 / magn;
                    if (Curvature > maxCurvature)
                        maxCurvature = Curvature;
                    if (Curvature < minCurvature)
                        minCurvature = Curvature;
                    curvatureArray.push(Curvature);
                    if (magn > 0.002)
                    {
                        var sd_vec = moi.vectorMath.createPoint((d_vec.x * sf / (magn * magn)), (d_vec.y * sf / (magn * magn)), (d_vec.z * sf / (magn * magn)));
                        scp = moi.vectorMath.createPoint(sd_vec.x + pts[n + 1].x, sd_vec.y + pts[n + 1].y, sd_vec.z + pts[n + 1].z);
                    } else
                    {
                        scp = moi.vectorMath.createPoint(pts[n + 1].x, pts[n + 1].y, pts[n + 1].z);
                    }
                    analysePts.push(scp);

                }
                /// Endpoint
                var ptsEndIndex = pts.length - 1;
                var d_vec = moi.vectorMath.createPoint(pts[ptsEndIndex].x - ptCend.x, pts[ptsEndIndex].y - ptCend.y, pts[ptsEndIndex].z - ptCend.z);
                var magn = magnitude(d_vec);
                var Curvature = 1 / magn;
                if (Curvature > maxCurvature)
                    maxCurvature = Curvature;
                if (Curvature < minCurvature)
                    minCurvature = Curvature;
                curvatureArray.push(Curvature);
                //   var sf = this.properties.scalefactor * 1000.0;
                if (magn > 0.002)
                {
                    var sd_vec = moi.vectorMath.createPoint((d_vec.x * sf / (magn * magn)), (d_vec.y * sf / (magn * magn)), (d_vec.z * sf / (magn * magn)));
                    scp = moi.vectorMath.createPoint(sd_vec.x + pts[ptsEndIndex].x, sd_vec.y + pts[ptsEndIndex].y, sd_vec.z + pts[ptsEndIndex].z);
                } else
                {
                    scp = moi.vectorMath.createPoint(pts[ptsEndIndex].x, pts[ptsEndIndex].y, pts[ptsEndIndex].z);
                }
                analysePts.push(scp);
            } else
            {/// Startpoint
                var d_vec = moi.vectorMath.createPoint(ptCstart.x - pts[0].x, ptCstart.y - pts[0].y, ptCstart.z - pts[0].z);
                var magnr = magnitude(d_vec);
                if (magnr < 10000)     //###################################
                {
                    var sd_vec = moi.vectorMath.createPoint(d_vec.x * this.properties.scalefactor, d_vec.y * this.properties.scalefactor, d_vec.z * this.properties.scalefactor);
                    var scp = moi.vectorMath.createPoint(sd_vec.x + pts[0].x, sd_vec.y + pts[0].y, sd_vec.z + pts[0].z);
                } else {
                    scp = moi.vectorMath.createPoint(pts[0].x, pts[0].y, pts[0].z);
                }
                analysePts.push(scp);
                //////// the normally calculated    
                for (var n = 0; n < ptC.length; n++)
                {
                    var d_vec = moi.vectorMath.createPoint(ptC[n].x - pts[n + 1].x, ptC[n].y - pts[n + 1].y, ptC[n].z - pts[n + 1].z);
                    magnr = magnitude(d_vec);
                    if (magnr < 10000)     //###################################
                    {
                        var sd_vec = moi.vectorMath.createPoint((d_vec.x * this.properties.scalefactor), (d_vec.y * this.properties.scalefactor), (d_vec.z * this.properties.scalefactor));
                        scp = moi.vectorMath.createPoint(sd_vec.x + pts[n + 1].x, sd_vec.y + pts[n + 1].y, sd_vec.z + pts[n + 1].z);
                    } else
                    {
                        scp = moi.vectorMath.createPoint(pts[n + 1].x, pts[n + 1].y, pts[n + 1].z);
                    }
                    analysePts.push(scp);

                }
                /// Endpoint
                var ptsEndIndex = pts.length - 1;
                var d_vec = moi.vectorMath.createPoint(ptCend.x - pts[ptsEndIndex].x, ptCend.y - pts[ptsEndIndex].y, ptCend.z - pts[ptsEndIndex].z);
                magnr = magnitude(d_vec);
                if (magnr < 10000)     //###################################
                {
                    var sd_vec = moi.vectorMath.createPoint((d_vec.x * this.properties.scalefactor), (d_vec.y * this.properties.scalefactor), (d_vec.z * this.properties.scalefactor));
                    scp = moi.vectorMath.createPoint(sd_vec.x + pts[ptsEndIndex].x, sd_vec.y + pts[ptsEndIndex].y, sd_vec.z + pts[ptsEndIndex].z);
                } else
                {
                    scp = moi.vectorMath.createPoint(pts[ptsEndIndex].x, pts[ptsEndIndex].y, pts[ptsEndIndex].z);
                }
                analysePts.push(scp);
            }
//            254 * (curvatureArray - minCurvature) / (maxCurvature - minCurvature);
            // create comp lines
            for (var i = 0; i < pts.length; i++)
            {
                var lineFac = moi.command.createFactory('line');
                lineFac.setInput(0, pts[i]);
                lineFac.setInput(1, analysePts[i]);
                var comp = lineFac.calculate();
                if (comp.length > 0)
                    output.addObject(comp.item(0));
                lineFac.cancel();
            }
            // create envelope curve

            var factory = moi.command.createFactory('interpcurve');
            for (var n = 0; n < analysePts.length; ++n)
            {
                factory.createInput('point');

                factory.setInput(factory.numInputs - 1, analysePts[n]);
            }
            var envelope = factory.calculate();
            output.addObject(envelope.item(0));
            this.setOutputData(0, output);
            factory.cancel();
        }

        ////set colors
        for (var n = 0; n < output.length - 1; n++)
        {
            output.item(n).styleIndex = this.rbID[Math.round(254 - (254 * (curvatureArray[n] - minCurvature) / (maxCurvature - minCurvature)))];
        }
        var minR=[1/maxCurvature];
        var maxR=[1/minCurvature];
        this.setOutputData(0, output);
        this.setOutputData(1, minR);
        this.setOutputData(2, maxR);
        this.outputs[1].label = "minR " + minR[0].toFixed(2); //.toFixed(2);
        this.outputs[2].label = "maxR " + maxR[0].toFixed(2); //.toFixed(2);
    }
    Curvature.prototype.onApply = function ()
    {
//        moi.geometryDatabase.removeObjects(this.tempObjects); // clear selectable Objects from a previous run
    }

    Curvature.prototype.onDrawBackground = function (ctx)
    {


    }
    LiteGraph.registerNodeType("Infos/Curvature", Curvature);
// ############ End Curvature ############

// ############ showFrame ############
    function showFrame()
    {
        this.addInput("In", "pointarray");
        this.addOutput("out", "objectlist");

    }

    showFrame.title = "showFrame";
    showFrame.desc = "showFrame";

    showFrame.prototype.onExecute = function ()
    {   var out=moi.geometryDatabase.createObjectList();
        var inpArray = this.getInputData(0, new pointArray(true));
        var inpLen=inpArray.getLength();
        if(inpLen>0)
            for(var i=0; i<inpLen;i++)
            {
            var sf = inpArray.getFrame(i);
            var ori = sf.origin;
            var x_a = sf.xaxis;
            var y_a = sf.yaxis;
            var z_a = sf.zaxis;
            var x1 = moi.VectorMath.createPoint(
                    ori.x + x_a.x,
                    ori.y + x_a.y,
                    ori.z + x_a.z
                    );
            var y1 = moi.VectorMath.createPoint(
                    ori.x + y_a.x,
                    ori.y + y_a.y,
                    ori.z + y_a.z
                    );

            var z1 = moi.VectorMath.createPoint(
                    ori.x + z_a.x,
                    ori.y + z_a.y,
                    ori.z + z_a.z
                    );
            var lfactory = moi.command.createFactory('line');
            lfactory.setInput(0, ori);
            lfactory.setInput(1, x1);
            var x = lfactory.calculate();
            lfactory.cancel();
            var lfactory = moi.command.createFactory('line');
            lfactory.setInput(0, ori);
            lfactory.setInput(1, y1);
            var y = lfactory.calculate();
            lfactory.cancel();

            var lfactory = moi.command.createFactory('line');
            lfactory.setInput(0, ori);
            lfactory.setInput(1, z1);
            var z = lfactory.calculate();
            lfactory.cancel();
            x.item(0).styleindex = 1;
            y.item(0).styleindex = 3;
            z.item(0).styleindex = 5;
            out.addObject(x.item(0));
            out.addObject(y.item(0));
            out.addObject(z.item(0));

        }
        this.outputs[0].label = "frame";
        this.setOutputData(0, out);

    }
    
    showFrame.prototype.onPropertyChange = function ()
    {

        //       this.outputs[0].label = this.bbox.xLength.toFixed(2);
        //       this.outputs[1].label = this.bbox.yLength.toFixed(2);
        //       this.outputs[2].label = this.bbox.zLength.toFixed(2);
    }

    LiteGraph.registerNodeType("Infos/showFrame", showFrame);


function GetBBoxes()
{
	this.addInput("In","objectlist");
	this.addInput("Elements","numarray");
	this.addInput("Step","numarray");	
	this.addOutput("Out","objectlist");
	this.addOutput("BBox Center","pointarray");
	this.properties = { elements:[0], step:[1] };

}

GetBBoxes.title = "BoundingBoxes";
GetBBoxes.desc = "Get BoundingBoxes";

GetBBoxes.prototype.onExecute = function()
{
	var input = this.getInputData(0, moi.geometryDatabase.createObjectList());
	var len = input.length;
	var bbox = new pointArray();
	
	this.properties.elements = this.getInputData(1, this.properties.elements);
	if ((this.properties.elements[0] <= 0) || (this.properties.elements[0] > len)) this.properties.elements[0] = len;
	this.properties.step = this.getInputData(2, this.properties.step);
	if (this.properties.step[0] <= 0) this.properties.step[0] = 1;
	if (this.properties.step[0] > len) this.properties.step[0] = len;
		
	this.setOutputData(0, input);
	
	for (i = 0; i < len && i < this.properties.elements[0]; i += this.properties.step[0]) {
		bbox.pushPoint(input.item(i).getBoundingBox().center);
	}
	this.setOutputData(1, bbox);
}

LiteGraph.registerNodeType("Infos/GetBBoxes", GetBBoxes);

    function stlVolume()
    {
        this.addInput("Body", "objectlist");
        this.addOutput("Volume", "numarray");
    }

    stlVolume.title = "stlVolume";
    stlVolume.desc = "stlVolume";
    stlVolume.prototype.onExecute = function () {
        var line = '', tmp_line = '', out = [], temp,
                vertex = [], volume = 0,i,
//                units = moi.geometryDatabase.unitsShortLabel,
                objects = this.getInputData(0, moi.geometryDatabase.createObjectList()),
                fileName = moi.settings.getIniPath().replace('moi.ini', 'volume_tmp.stl'),
                options = 'NoUI=true;Angle=6;stlExportFileType=Ascii';
        var OldSTLExportType = moi.settings.stlExportFileType;
        moi.settings.stlExportFileType = 'Ascii';


        for (i = 0; i < objects.length; i++) {
            temp = objects.item(i).clone();
            temp.name = "temp";
            moi.geometryDatabase.addObject(temp);
            moi.geometryDatabase.selectNamed("temp");
            volume = 0;
            moi.geometryDatabase.fileExport(fileName, options);
            var file = moi.filesystem.openFileStream(fileName, 'r');

            while (!file.AtEOF) {
                tmp_line = file.readLine().trim();
                if (tmp_line[0] === 'v') {
                    line = tmp_line.split(' ');
                    for (var j = 1; j < 4; j++) {
                        vertex[j] = {x: line[1] * 1, y: line[2] * 1, z: line[3] * 1}
                        line = file.readLine().trim().split(' ');
                    }
                    volume += (
                            vertex[1].x * vertex[2].y * vertex[3].z -
                            vertex[1].x * vertex[3].y * vertex[2].z -
                            vertex[2].x * vertex[1].y * vertex[3].z +
                            vertex[2].x * vertex[3].y * vertex[1].z +
                            vertex[3].x * vertex[1].y * vertex[2].z -
                            vertex[3].x * vertex[2].y * vertex[1].z
                            ) / 6;
//                    moi.ui.alert(volume);
//        var l1=  factory('line',moi.vectorMath.createPoint(vertex[1].x,vertex[1].y,vertex[1].z),moi.vectorMath.createPoint(vertex[2].x,vertex[2].y,vertex[2].z));
//         moi.geometryDatabase.addObjects(l1);
                }
            }
            moi.geometryDatabase.removeObject(temp);
            temp = moi.geometryDatabase.createObjectList();
            out.push(volume);
        }
        this.setOutputData(0, out);
    }
    LiteGraph.registerNodeType("Infos/stlVolume", stlVolume);
    
    function stlArea()
    {
        this.addInput("BRep", "objectlist");
        this.addOutput("Area", "numarray");
    }

    stlArea.title = "stlArea";
    stlArea.desc = "stlArea";
    stlArea.prototype.onExecute = function () {
        var line = '', tmp_line = '', out = [], temp,
                vertex = [], area = 0,i,
//                units = moi.geometryDatabase.unitsShortLabel,
                objects = this.getInputData(0, moi.geometryDatabase.createObjectList()),
                fileName = moi.settings.getIniPath().replace('moi.ini', 'volume_tmp.stl'),
                options = 'NoUI=true;Angle=6;stlExportFileType=Ascii';
        var OldSTLExportType = moi.settings.stlExportFileType;
        moi.settings.stlExportFileType = 'Ascii';
//        var objlen=objects.length;
//        if(objlen>0){
        for (i = 0; i < objects.length; i++) {
            temp = objects.item(i).clone();
            temp.name = "temp";
            moi.geometryDatabase.addObject(temp);
            moi.geometryDatabase.selectNamed("temp");
            area = 0;
            moi.geometryDatabase.fileExport(fileName, options);
            var file = moi.filesystem.openFileStream(fileName, 'r');

            while (!file.AtEOF) {
                tmp_line = file.readLine().trim();
                if (tmp_line[0] === 'v') {
                    line = tmp_line.split(' ');
                    for (var j = 1; j < 4; j++) {
                        vertex[j] = {x: line[1] * 1, y: line[2] * 1, z: line[3] * 1}
                        line = file.readLine().trim().split(' ');
                    }
                    area += magnitude(crossProd(substrVec(vertex[2],vertex[1]),substrVec(vertex[3],vertex[1])))/2;
//                    moi.ui.alert(volume);
//        var l1=  factory('line',moi.vectorMath.createPoint(vertex[1].x,vertex[1].y,vertex[1].z),moi.vectorMath.createPoint(vertex[2].x,vertex[2].y,vertex[2].z));
//         moi.geometryDatabase.addObjects(l1);
                }
            }
            moi.geometryDatabase.removeObject(temp);
            temp = moi.geometryDatabase.createObjectList();
            out.push(area);
        }
//    }
        this.setOutputData(0, out);
    }
    LiteGraph.registerNodeType("Infos/stlArea", stlArea);
    
    //
        function camera()
{

	this.addInput("camPt","pointarray");
	this.addInput("target","pointarray");
        this.properties = {encoder:"",filePath:"/home/jones/pngs/",Width:800,Height:600,save:["Off","Off","On"],idx:0,frames:100};
}

camera.title = "camera";
camera.desc = "camera";

    camera.prototype.getExtraMenuOptions = function ()
    {
        var that = this;
        return [
            {content: lang.getTranslation("Load"), callback: function () {
                    that.properties.filename = moi.filesystem.getOpenFileName('Choose a CSV file', 'Point files (*.txt, *.csv)|*.txt;*.csv|All files (*.*)|*.*');
                }}
        ];
    }

    camera.prototype.onAdded = function ()
    {
        if (this.properties.encoder === "")
           this.properties.encoder = moi.filesystem.getOpenFileName('Choose a CSV file', 'Point files (*.txt, *.csv)|*.txt;*.csv|All files (*.*)|*.*');
    }
camera.prototype.onExecute = function()
{         camPt = new pointArray(false);
          camPt.push(100,100,100,0,0,0,1);
            
        var camPtA = this.getInputData(0, camPt);
        var targetPtA = this.getInputData(1, new pointArray(true));
        var camPt=camPtA.getPoint(0);
        var targetPt=targetPtA.getPoint(0);
        var Msg="Cpt";Msg += moi.ui.getActiveViewport().cameraPt.x;Msg +=" ";Msg += moi.ui.getActiveViewport().cameraPt.y;Msg +=" ";Msg += moi.ui.getActiveViewport().cameraPt.z;
//        moi.UI.alert(Msg);
        moi.ui.getActiveViewport().reset();
        moi.ui.getActiveViewport().setCameraAndTarget(camPt,targetPt);
        moi.ui.getActiveViewport().zoom(1.0);
        if(this.properties.save[0]==="On"&&this.properties.idx<this.properties.frames){
//            var filePath="/home/jones/pngs/";
        this.properties.idx+=1;
        var fname="test";
        var adress=this.properties.filePath;adress+=fname;adress+=this.properties.idx;adress+=".png";
        moi.ui.getActiveViewport().render( this.properties.Width, this.properties.Height).save(adress);
//        moi.UI.alert(adress);
    }
//    if(this.properties.idx>this.properties.frames){moi.filesystem.shellExecute('/usr/bin/ls -t /home/jones/pngs/*.png > /home/jones/pngs/file-list');moi.filesystem.shellExecute('/usr/bin/mencoder mf:///home/jones/@file-list -mf w=800:h=600:fps=25:type=png -ovc lavc -lavcopts vcodec=mpeg4:mbd=2:trell -oac copy -o /home/jones/pngs/output.avi');}
//    if(this.properties.idx>this.properties.frames)moi.filesystem.shellExecute('/home/jones/pngs/coder');
    if(this.properties.idx>this.properties.frames){moi.filesystem.shellExecute('/usr/bin/ffmpeg -r 25 -i /home/jones/pngs/test%000d.png -c:v libx264 /home/jones/pngs/out.avi');
// ffmpeg -r 25 -i /home/jones/pngs/test%d.png -c:v libx264 out.avi
}
}

camera.prototype.onAdded = function ()
    {	

    }
LiteGraph.registerNodeType("Infos/camera", camera);

    //
})();

