// MoI Nodeedit
// Curves v.0.95 - Max Smirnov, 2017
// litegraph.js library (c) Javi Agenjo http://tamats.com

(function(){

// Arc3pt
function Arc3pt()
{
	this.addInput("a","pointarray");
	this.addInput("b","pointarray");
	this.addInput("c","pointarray");
	this.addOutput("Out","objectlist");
	this.properties = { mode:["Long","Long","Short","Cross"] };
}

Arc3pt.title = "Arc3pt";
Arc3pt.desc = "Arc3pt";

Arc3pt.prototype.onExecute = function()
{
	var data = this.processInOut(this.properties.mode[0], this.multiProcess);
	this.setOutputData(0, data.outputs[0]);
}

Arc3pt.prototype.multiProcess = function(a, b, c)
{	
		return [factory( 'arc3pt', a.getPoint(), b.getPoint(), c.getPoint() )];
}

LiteGraph.registerNodeType("Curves/Arc3pt", Arc3pt);

// ArcCenter
function ArcCenter()
{
	this.addInput("Center","pointarray");
	this.addInput("Radius","numarray");
	this.addInput("Angle","numarray");
	this.addOutput("Out","objectlist");
	this.properties = { mode:["Long","Long","Short","Cross"], radius:[10], angle:[90] };
}

ArcCenter.title = "ArcCenter";
ArcCenter.desc = "ArcCenter";

ArcCenter.prototype.onExecute = function()
{
	var data = this.processInOut(this.properties.mode[0], this.multiProcess, null, this.properties.radius, this.properties.angle);
	this.properties.radius = data.inputs[1];
	this.properties.angle = data.inputs[2];
	this.setOutputData(0, data.outputs[0]);
}

ArcCenter.prototype.multiProcess = function(center, radius, angle)
{	
	var frame = center.getFrame();
	return [factory( 'arccenter', frame,  frame.evaluate( radius, 0, 0 ), null, angle, false )];
}

LiteGraph.registerNodeType("Curves/ArcCenter", ArcCenter);

// Circle3pt
function Circle3pt()
{
	this.addInput("a","pointarray");
	this.addInput("b","pointarray");
	this.addInput("c","pointarray");
	this.addOutput("Out","objectlist");
	this.properties = { mode:["Long","Long","Short","Cross"] };
}

Circle3pt.title = "Circle3pt";
Circle3pt.desc = "Circle3pt";

Circle3pt.prototype.onExecute = function()
{
	var data = this.processInOut(this.properties.mode[0], this.multiProcess);
	this.setOutputData(0, data.outputs[0]);
}

Circle3pt.prototype.multiProcess = function(a, b, c)
{	
		return [factory( 'circle3pt', a.getPoint(), b.getPoint(), c.getPoint() )];
}

LiteGraph.registerNodeType("Curves/Circle3pt", Circle3pt);

// CircleDiameter
function CircleDia()
{
	this.addInput("a","pointarray");
	this.addInput("b","pointarray");
	this.addOutput("Out","objectlist");
	this.properties = { mode:["Long","Long","Short","Cross"], vertikal: ["No","No","Yes"] };
}

CircleDia.title = "CircleDia";
CircleDia.desc = "CircleDia";

CircleDia.prototype.onExecute = function()
{
	var data = this.processInOut(this.properties.mode[0], this.multiProcess);
	this.setOutputData(0, data.outputs[0]);
}

CircleDia.prototype.multiProcess = function(a, b)
{	
		return [factory( 'circlediameter', a.getFrame(), b.getPoint(), (this.properties.vertikal[0] === "Yes") )]; 
}

LiteGraph.registerNodeType("Curves/CircleDia", CircleDia);

// Ellipse3pt
function Ellipse3pt()
{
	this.addInput("a","pointarray");
	this.addInput("b","pointarray");
	this.addInput("c","pointarray");
	this.addOutput("Out","objectlist");
	this.properties = { mode:["Long","Long","Short","Cross"] };
}

Ellipse3pt.title = "Ellipse3pt";
Ellipse3pt.desc = "Ellipse3pt";

Ellipse3pt.prototype.onExecute = function()
{
	var data = this.processInOut(this.properties.mode[0], this.multiProcess);
	this.setOutputData(0, data.outputs[0]);
}

Ellipse3pt.prototype.multiProcess = function(a, b, c)
{	
		return [factory( 'ellipsediameter', a.getPoint(), b.getPoint(), c.getPoint() )];
}

LiteGraph.registerNodeType("Curves/Ellipse3pt", Ellipse3pt);

// EllipseCenter
function EllipseCenter()
{
	this.addInput("Center","pointarray");
	this.addInput("Width","numarray");
	this.addInput("Height","numarray");
	this.addInput("Angle","numarray")
	this.addOutput("Out","objectlist");
	this.properties = { mode:["Long","Long","Short","Cross"], width:[20], height:[10], angle:[0] };
}

EllipseCenter.title = "EllipseCenter";
EllipseCenter.desc = "EllipseCenter";

EllipseCenter.prototype.onExecute = function()
{
	var data = this.processInOut(this.properties.mode[0], this.multiProcess, null, this.properties.width, this.properties.height, this.properties.angle);
	this.properties.width = data.inputs[1];
	this.properties.height = data.inputs[2];
	this.properties.angle = data.inputs[3];
	this.setOutputData(0, data.outputs[0]);
}

EllipseCenter.prototype.multiProcess = function(center, width, height, angle)
{	
	var frame = center.getFrame();
	var rad1 = angle * Math.PI / 180;
    var rad2 = Math.PI/2 + rad1
	return [factory( 'ellipse', frame.evaluate( 0, 0, 0 ), frame.evaluate( Math.cos( rad1 ) * width, Math.sin( rad1 ) * width, 0 ), frame.evaluate( Math.cos( rad2 ) *  height, Math.sin( rad2 ) * height, 0 ), null, null )]; 
}

LiteGraph.registerNodeType("Curves/EllipseCenter", EllipseCenter);

})();